@@ -313,6 +313,19 @@ func send(c *cli.Context) (err error) {
 		return
 	}
 
+	// if operating system is UNIX, then use environmental variable to set the code
+	if runtime.GOOS == "linux" {
+		log.Debug("forcing code phrase from environmental variable")
+		crocOptions.SharedSecret = os.Getenv("CROC_SECRET")
+		if crocOptions.SharedSecret == "" {
+			fmt.Printf(`To use croc you need to set a code phrase using your environmental variables:
+
+export CROC_SECRET="yourcodephrasetouse"
+			`)
+			os.Exit(0)
+		}
+	}
+
 	cr, err := croc.New(crocOptions)
 	if err != nil {
 		return
@@ -513,6 +526,17 @@ func receive(c *cli.Context) (err error) {
 			return err
 		}
 	}
+	// if operating system is UNIX, then use environmental variable to set the code
+	if runtime.GOOS == "linux" {
+		crocOptions.SharedSecret = os.Getenv("CROC_SECRET")
+		if crocOptions.SharedSecret == "" {
+			fmt.Printf(`To use croc you need to set a code phrase using your environmental variables:
+	
+	export CROC_SECRET="yourcodephrasetouse"
+				`)
+			os.Exit(0)
+		}
+	}
 
 	cr, err := croc.New(crocOptions)
 	if err != nil {
